@@ -19,45 +19,37 @@
  * along with this library; if not, see <http://www.gnu.org/licenses>.
  */
 
+#include <elf.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
 #include <asm/bitsperlong.h>
 #include <linux/audit.h>
+#include <stdbool.h>
 
 #include <seccomp.h>
 
 #include "arch.h"
-#include "arch-i386.h"
+#include "arch-x86.h"
 #include "arch-x86_64.h"
+#include "arch-x32.h"
+#include "arch-arm.h"
 #include "system.h"
 
-const struct arch_def arch_def_native = {
 #if __i386__
-	.token = AUDIT_ARCH_I386,
+const struct arch_def *arch_def_native = &arch_def_x86;
 #elif __x86_64__
-	.token = AUDIT_ARCH_X86_64,
+#ifdef __ILP32__
+const struct arch_def *arch_def_native = &arch_def_x32;
+#else
+const struct arch_def *arch_def_native = &arch_def_x86_64;
+#endif /* __ILP32__ */
+#elif __arm__
+const struct arch_def *arch_def_native = &arch_def_arm;
 #else
 #error the arch code needs to know about your machine type
 #endif /* machine type guess */
 
-#if __BITS_PER_LONG == 32
-	.size = ARCH_SIZE_32,
-#elif __BITS_PER_LONG == 64
-	.size = ARCH_SIZE_64,
-#else
-	.size = ARCH_SIZE_UNSPEC,
-#endif /* BITS_PER_LONG */
-
-#if __BYTE_ORDER == __LITTLE_ENDIAN
-	.endian = ARCH_ENDIAN_LITTLE,
-#elif __BYTE_ORDER == __BIG_ENDIAN
-	.endian = ARCH_ENDIAN_BIG,
-#else
-	.endian = ARCH_ENDIAN_UNSPEC,
-#endif /* __BYTE_ORDER */
-};
-
 /**
  * Validate the architecture token
  * @param arch the architecture token
@@ -68,35 +60,16 @@ const struct arch_def arch_def_native = {
 int arch_valid(uint32_t arch)
 {
 	switch (arch) {
-	case AUDIT_ARCH_I386:
-	case AUDIT_ARCH_X86_64:
+	case SCMP_ARCH_X86:
+	case SCMP_ARCH_X86_64:
+	case SCMP_ARCH_X32:
+	case SCMP_ARCH_ARM:
 		return 0;
 	}
 
 	return -EINVAL;
 }
 
-/**
- * Lookup the syscall table for an architecture
- * @param token the architecure token
- *
- * Return the architecture's syscall table, returns NULL on failure.
- *
- */
-static const struct arch_syscall_def *_arch_syscall_lookup(uint32_t token)
-{
-	switch (token) {
-	case AUDIT_ARCH_I386:
-		return i386_syscall_table;
-		break;
-	case AUDIT_ARCH_X86_64:
-		return x86_64_syscall_table;
-		break;
-	}
-
-	return NULL;
-}
-
 /**
  * Lookup the architecture definition
  * @param token the architecure token
@@ -107,12 +80,14 @@ static const struct arch_syscall_def *_arch_syscall_lookup(uint32_t token)
 const struct arch_def *arch_def_lookup(uint32_t token)
 {
 	switch (token) {
-	case AUDIT_ARCH_I386:
-		return &arch_def_i386;
-		break;
-	case AUDIT_ARCH_X86_64:
+	case SCMP_ARCH_X86:
+		return &arch_def_x86;
+	case SCMP_ARCH_X86_64:
 		return &arch_def_x86_64;
-		break;
+	case SCMP_ARCH_X32:
+		return &arch_def_x32;
+	case SCMP_ARCH_ARM:
+		return &arch_def_arm;
 	}
 
 	return NULL;
@@ -129,13 +104,17 @@ const struct arch_def *arch_def_lookup(uint32_t token)
 int arch_arg_count_max(const struct arch_def *arch)
 {
 	switch (arch->token) {
-	case AUDIT_ARCH_I386:
-		return i386_arg_count_max;
-	case AUDIT_ARCH_X86_64:
+	case SCMP_ARCH_X86:
+		return x86_arg_count_max;
+	case SCMP_ARCH_X86_64:
 		return x86_64_arg_count_max;
-	default:
-		return -EDOM;
+	case SCMP_ARCH_X32:
+		return x32_arg_count_max;
+	case SCMP_ARCH_ARM:
+		return arm_arg_count_max;
 	}
+
+	return -EDOM;
 }
 
 /**
@@ -151,7 +130,7 @@ int arch_arg_count_max(const struct arch_def *arch)
 int arch_arg_offset_lo(const struct arch_def *arch, unsigned int arg)
 {
 	switch (arch->token) {
-	case AUDIT_ARCH_X86_64:
+	case SCMP_ARCH_X86_64:
 		return x86_64_arg_offset_lo(arg);
 	default:
 		return -EDOM;
@@ -171,7 +150,7 @@ int arch_arg_offset_lo(const struct arch_def *arch, unsigned int arg)
 int arch_arg_offset_hi(const struct arch_def *arch, unsigned int arg)
 {
 	switch (arch->token) {
-	case AUDIT_ARCH_X86_64:
+	case SCMP_ARCH_X86_64:
 		return x86_64_arg_offset_hi(arg);
 	default:
 		return -EDOM;
@@ -185,22 +164,20 @@ int arch_arg_offset_hi(const struct arch_def *arch, unsigned int arg)
  *
  * Resolve the given syscall name to the syscall number based on the given
  * architecture.  Returns the syscall number on success, including negative
- * pseudo syscall numbers; returns -1 on failure.
+ * pseudo syscall numbers; returns __NR_SCMP_ERROR on failure.
  *
  */
 int arch_syscall_resolve_name(const struct arch_def *arch, const char *name)
 {
-	unsigned int iter;
-	const struct arch_syscall_def *table;
-
-	table = _arch_syscall_lookup(arch->token);
-	if (table == NULL)
-		return __NR_SCMP_ERROR;
-
-	/* XXX - plenty of room for future improvement here */
-	for (iter = 0; table[iter].name != NULL; iter++) {
-		if (strcmp(name, table[iter].name) == 0)
-			return table[iter].num;
+	switch (arch->token) {
+	case SCMP_ARCH_X86:
+		return x86_syscall_resolve_name(name);
+	case SCMP_ARCH_X86_64:
+		return x86_64_syscall_resolve_name(name);
+	case SCMP_ARCH_X32:
+		return x32_syscall_resolve_name(name);
+	case SCMP_ARCH_ARM:
+		return arm_syscall_resolve_name(name);
 	}
 
 	return __NR_SCMP_ERROR;
@@ -218,17 +195,15 @@ int arch_syscall_resolve_name(const struct arch_def *arch, const char *name)
  */
 const char *arch_syscall_resolve_num(const struct arch_def *arch, int num)
 {
-	unsigned int iter;
-	const struct arch_syscall_def *table;
-
-	table = _arch_syscall_lookup(arch->token);
-	if (table == NULL)
-		return NULL;
-
-	/* XXX - plenty of room for future improvement here */
-	for (iter = 0; table[iter].num != __NR_SCMP_ERROR; iter++) {
-		if (num == table[iter].num)
-			return table[iter].name;
+	switch (arch->token) {
+	case SCMP_ARCH_X86:
+		return x86_syscall_resolve_num(num);
+	case SCMP_ARCH_X86_64:
+		return x86_64_syscall_resolve_num(num);
+	case SCMP_ARCH_X32:
+		return x32_syscall_resolve_num(num);
+	case SCMP_ARCH_ARM:
+		return arm_syscall_resolve_num(num);
 	}
 
 	return NULL;
@@ -249,8 +224,8 @@ int arch_syscall_translate(const struct arch_def *arch, int *syscall)
 	int sc_num;
 	const char *sc_name;
 
-	if (arch->token != arch_def_native.token) {
-		sc_name = arch_syscall_resolve_num(&arch_def_native, *syscall);
+	if (arch->token != arch_def_native->token) {
+		sc_name = arch_syscall_resolve_num(arch_def_native, *syscall);
 		if (sc_name == NULL)
 			return -EFAULT;
 
@@ -267,21 +242,41 @@ int arch_syscall_translate(const struct arch_def *arch, int *syscall)
 /**
  * Rewrite a syscall value to match the architecture
  * @param arch the architecture definition
+ * @param strict strict flag
  * @param syscall the syscall number
  *
  * Syscalls can vary across different architectures so this function rewrites
- * the syscall into the correct value for the specified architecture.  Returns
- * zero on success, negative values on failure.
+ * the syscall into the correct value for the specified architecture.  If
+ * @strict is true then the function will fail if the syscall can not be
+ * preservered, however, if @strict is false the function will do a "best
+ * effort" rewrite and not fail. Returns zero on success, negative values on
+ * failure.
  *
  */
-int arch_syscall_rewrite(const struct arch_def *arch, int *syscall)
+int arch_syscall_rewrite(const struct arch_def *arch, bool strict, int *syscall)
 {
-	switch (arch->token) {
-	case AUDIT_ARCH_I386:
-		return i386_syscall_rewrite(arch, syscall);
-	default:
-		return -EDOM;
+	int sys = *syscall;
+
+	if (sys >= 0) {
+		/* we shouldn't be here - no rewrite needed */
+		return 0;
+	} else if (sys < 0 && sys > -100) {
+		/* reserved values */
+		return -EINVAL;
+	} else if (sys <= -100 && sys > -10000) {
+		/* rewritable syscalls */
+		switch (arch->token) {
+		case SCMP_ARCH_X86:
+			return x86_syscall_rewrite(arch, strict, syscall);
+		}
+		/* NOTE: we fall through to the default handling (strict?) if
+		 *       we don't support any rewriting for the architecture */
 	}
+
+	/* syscalls not defined on this architecture */
+	if (strict)
+		return -EDOM;
+	return 0;
 }
 
 /**
@@ -300,13 +295,28 @@ int arch_syscall_rewrite(const struct arch_def *arch, int *syscall)
  *
  */
 int arch_filter_rewrite(const struct arch_def *arch,
-			unsigned int strict,
-			int *syscall, struct db_api_arg *chain)
+			bool strict, int *syscall, struct db_api_arg *chain)
 {
-	switch (arch->token) {
-	case AUDIT_ARCH_I386:
-		return i386_filter_rewrite(arch, strict, syscall, chain);
-	default:
-		return -EDOM;
+	int sys = *syscall;
+
+	if (sys >= 0) {
+		/* we shouldn't be here - no rewrite needed */
+		return 0;
+	} else if (sys < 0 && sys > -100) {
+		/* reserved values */
+		return -EINVAL;
+	} else if (sys <= -100 && sys > -10000) {
+		/* rewritable syscalls */
+		switch (arch->token) {
+		case SCMP_ARCH_X86:
+			return x86_filter_rewrite(arch, strict, syscall, chain);
+		}
+		/* NOTE: we fall through to the default handling (strict?) if
+		 *       we don't support any rewriting for the architecture */
 	}
+
+	/* syscalls not defined on this architecture */
+	if (strict)
+		return -EDOM;
+	return 0;
 }
