@@ -24,6 +24,7 @@
 
 #include <inttypes.h>
 #include <stddef.h>
+#include <stdbool.h>
 
 #include <seccomp.h>
 
@@ -33,6 +34,7 @@ struct db_api_arg;
 
 struct arch_def {
 	uint32_t token;
+	uint32_t token_bpf;
 	enum {
 		ARCH_SIZE_UNSPEC = 0,
 		ARCH_SIZE_32 = 32,
@@ -46,7 +48,7 @@ struct arch_def {
 };
 
 /* arch_def for the current architecture */
-extern const struct arch_def arch_def_native;
+extern const struct arch_def *arch_def_native;
 
 /* NOTE: Syscall mappings can be found by running the following commands
  *	 on the specific architecture's include file:
@@ -95,10 +97,10 @@ int arch_syscall_resolve_name(const struct arch_def *arch, const char *name);
 const char *arch_syscall_resolve_num(const struct arch_def *arch, int num);
 
 int arch_syscall_translate(const struct arch_def *arch, int *syscall);
-int arch_syscall_rewrite(const struct arch_def *arch, int *syscall);
+int arch_syscall_rewrite(const struct arch_def *arch, bool strict,
+			 int *syscall);
 
 int arch_filter_rewrite(const struct arch_def *arch,
-			unsigned int strict,
-			int *syscall, struct db_api_arg *chain);
+			bool strict, int *syscall, struct db_api_arg *chain);
 
 #endif
