@@ -89,8 +89,6 @@ static unsigned int _db_tree_free(struct db_arg_chain_tree *tree)
  * Remove a node from an argument chain tree
  * @param tree the pointer to the tree
  * @param node the node to remove
- * @param action_flg the replacement action flag
- * @param action the replacement action
  *
  * This function searches the tree looking for the node and removes it once
  * found.  The function also removes any other nodes that are no longer needed
@@ -98,9 +96,7 @@ static unsigned int _db_tree_free(struct db_arg_chain_tree *tree)
  *
  */
 static unsigned int _db_tree_remove(struct db_arg_chain_tree **tree,
-				    struct db_arg_chain_tree *node,
-				    unsigned int action_flg,
-				    uint32_t action)
+				    struct db_arg_chain_tree *node)
 {
 	int cnt = 0;
 	struct db_arg_chain_tree *c_iter;
@@ -134,10 +130,8 @@ static unsigned int _db_tree_remove(struct db_arg_chain_tree **tree,
 		}
 
 		/* check the true/false sub-trees */
-		cnt += _db_tree_remove(&(c_iter->nxt_t), node,
-				       action_flg, action);
-		cnt += _db_tree_remove(&(c_iter->nxt_f), node,
-				       action_flg, action);
+		cnt += _db_tree_remove(&(c_iter->nxt_t), node);
+		cnt += _db_tree_remove(&(c_iter->nxt_f), node);
 
 		c_iter = c_iter->lvl_nxt;
 	} while (c_iter != NULL);
@@ -201,89 +195,57 @@ static int _db_tree_act_check(struct db_arg_chain_tree *tree, uint32_t action)
 static int _db_tree_sub_prune(struct db_arg_chain_tree **tree_head,
 			      struct db_arg_chain_tree *tree_start,
 			      struct db_arg_chain_tree *new,
-			      unsigned int *remove_flg)
+			      bool *remove_flg)
 {
 	int rc = 0;
 	struct db_arg_chain_tree *c_iter = tree_start;
 
-	*remove_flg = 0;
+	*remove_flg = false;
 
 	if (new == NULL || c_iter == NULL)
 		return 0;
-	if (!db_chain_one_result(new))
-		return 0;
 
 	while (c_iter->lvl_prv != NULL)
 		c_iter = c_iter->lvl_prv;
-
 	do {
-		if (c_iter->arg < new->arg) {
-			if (c_iter->nxt_t != NULL) {
-				rc = _db_tree_sub_prune(tree_head,
-							c_iter->nxt_t, new,
-							remove_flg);
-				if (rc > 0)
-					goto sub_prune_found_down;
-			}
-			if (c_iter->nxt_f != NULL) {
-				rc = _db_tree_sub_prune(tree_head,
-							c_iter->nxt_f, new,
-							remove_flg);
-				if (rc > 0)
-					goto sub_prune_found_down;
-			}
-		} else if (db_chain_eq(c_iter, new)) {
-			if (db_chain_leaf(new)) {
-				if (!db_chain_one_result(c_iter)) {
-					/* can't remove existing node */
-					if (new->act_t_flg && c_iter->act_t_flg
-					    && new->act_t == c_iter->act_t)
-						c_iter->act_t_flg = 0;
-					if (new->act_f_flg && c_iter->act_f_flg
-					    && new->act_f == c_iter->act_f)
-						c_iter->act_f_flg = 0;
-					goto sub_prune_return;
-				}
-				if ((db_chain_eq_result(c_iter, new)) ||
-				    ((new->act_t_flg && c_iter->nxt_t != NULL)||
-				     (new->act_f_flg && c_iter->nxt_f != NULL)))
-					/* exact or close match - remove node */
-					goto sub_prune_remove;
-				goto sub_prune_return;
-			}
-			if (new->nxt_t != NULL) {
-				rc = _db_tree_sub_prune(tree_head,
-							c_iter->nxt_t,
-							new->nxt_t,
-							remove_flg);
-				if (rc > 0)
-					goto sub_prune_found_down;
-			}
-			if (new->nxt_f != NULL) {
-				rc = _db_tree_sub_prune(tree_head,
-							c_iter->nxt_f,
-							new->nxt_f,
-							remove_flg);
-				if (rc > 0)
-					goto sub_prune_found_down;
-			}
+		if (db_chain_eq(c_iter, new)) {
+			if (new->act_t_flg) {
+				rc += _db_tree_remove(tree_head, c_iter->nxt_t);
+				c_iter->act_t = new->act_t;
+				c_iter->act_t_flg = true;
+			} else if (new->nxt_t != NULL)
+				rc += _db_tree_sub_prune(tree_head,
+							 c_iter->nxt_t,
+							 new->nxt_t,
+							 remove_flg);
+			if (new->act_f_flg) {
+				rc += _db_tree_remove(tree_head, c_iter->nxt_f);
+				c_iter->act_f = new->act_f;
+				c_iter->act_f_flg = true;
+			} else if (new->nxt_f != NULL)
+				rc += _db_tree_sub_prune(tree_head,
+							 c_iter->nxt_f,
+							 new->nxt_f,
+							 remove_flg);
+		} else if (db_chain_lt(c_iter, new)) {
+			if (c_iter->nxt_t != NULL)
+				rc += _db_tree_sub_prune(tree_head,
+							 c_iter->nxt_t, new,
+							 remove_flg);
+			if (c_iter->nxt_f != NULL)
+				rc += _db_tree_sub_prune(tree_head,
+							 c_iter->nxt_f, new,
+							 remove_flg);
+
 		} else if (db_chain_gt(c_iter, new))
 			goto sub_prune_return;
 
 		c_iter = c_iter->lvl_nxt;
 	} while (c_iter != NULL);
 
-	goto sub_prune_return;
-
-sub_prune_found_down:
-	if (db_chain_zombie(c_iter))
-		goto sub_prune_remove;
 sub_prune_return:
-	*remove_flg = 0;
-	return rc;
-sub_prune_remove:
-	rc += _db_tree_remove(tree_head, c_iter, 0, 0);
-	*remove_flg = 1;
+	if (rc > 0)
+		*remove_flg = true;
 	return rc;
 }
 
@@ -717,7 +679,7 @@ int db_syscall_priority(struct db_filter *db,
 	memset(s_new, 0, sizeof(*s_new));
 	s_new->num = syscall;
 	s_new->priority = sys_pri;
-	s_new->valid = 0;
+	s_new->valid = false;
 
 	/* add it before s_iter */
 	if (s_prev != NULL) {
@@ -768,15 +730,15 @@ static struct db_sys_list *_db_rule_gen_64(const struct arch_def *arch,
 	int chain_len_max;
 	struct db_sys_list *s_new;
 	struct db_arg_chain_tree *c_iter_hi = NULL, *c_iter_lo = NULL;
-	struct db_arg_chain_tree *c_prev_lo = NULL;
-	unsigned int tf_flag;
+	struct db_arg_chain_tree *c_prev_hi = NULL, *c_prev_lo = NULL;
+	bool tf_flag;
 
 	s_new = malloc(sizeof(*s_new));
 	if (s_new == NULL)
 		return NULL;
 	memset(s_new, 0, sizeof(*s_new));
 	s_new->num = syscall;
-	s_new->valid = 1;
+	s_new->valid = true;
 	/* run through the argument chain */
 	chain_len_max = arch_arg_count_max(arch);
 	for (iter = 0; iter < chain_len_max; iter++) {
@@ -798,10 +760,12 @@ static struct db_sys_list *_db_rule_gen_64(const struct arch_def *arch,
 
 		/* link this level to the previous level */
 		if (c_prev_lo != NULL) {
-			if (tf_flag)
-				c_prev_lo->nxt_t = c_iter_hi;
-			else
+			if (!tf_flag) {
 				c_prev_lo->nxt_f = c_iter_hi;
+				c_prev_hi->nxt_f = c_iter_hi;
+				c_iter_hi->refcnt++;
+			} else
+				c_prev_lo->nxt_t = c_iter_hi;
 		} else
 			s_new->chains = c_iter_hi;
 		s_new->node_cnt += 2;
@@ -817,27 +781,27 @@ static struct db_sys_list *_db_rule_gen_64(const struct arch_def *arch,
 			case SCMP_CMP_GT:
 				c_iter_hi->op = SCMP_CMP_GE;
 				c_iter_lo->op = SCMP_CMP_GT;
-				tf_flag = 1;
+				tf_flag = true;
 				break;
 			case SCMP_CMP_NE:
 				c_iter_hi->op = SCMP_CMP_EQ;
 				c_iter_lo->op = SCMP_CMP_EQ;
-				tf_flag = 0;
+				tf_flag = false;
 				break;
 			case SCMP_CMP_LT:
 				c_iter_hi->op = SCMP_CMP_GE;
 				c_iter_lo->op = SCMP_CMP_GE;
-				tf_flag = 0;
+				tf_flag = false;
 				break;
 			case SCMP_CMP_LE:
 				c_iter_hi->op = SCMP_CMP_GE;
 				c_iter_lo->op = SCMP_CMP_GT;
-				tf_flag = 0;
+				tf_flag = false;
 				break;
 			default:
 				c_iter_hi->op = chain[iter].op;
 				c_iter_lo->op = chain[iter].op;
-				tf_flag = 1;
+				tf_flag = true;
 		}
 		c_iter_hi->mask = D64_HI(chain[iter].mask);
 		c_iter_lo->mask = D64_LO(chain[iter].mask);
@@ -851,16 +815,19 @@ static struct db_sys_list *_db_rule_gen_64(const struct arch_def *arch,
 		/* link the hi and lo chain nodes */
 		c_iter_hi->nxt_t = c_iter_lo;
 
+		c_prev_hi = c_iter_hi;
 		c_prev_lo = c_iter_lo;
 	}
 	if (c_iter_lo != NULL) {
 		/* set the leaf node */
-		if (tf_flag) {
-			c_iter_lo->act_t_flg = 1;
-			c_iter_lo->act_t = action;
-		} else {
-			c_iter_lo->act_f_flg = 1;
+		if (!tf_flag) {
+			c_iter_lo->act_f_flg = true;
 			c_iter_lo->act_f = action;
+			c_iter_hi->act_f_flg = true;
+			c_iter_hi->act_f = action;
+		} else {
+			c_iter_lo->act_t_flg = true;
+			c_iter_lo->act_t = action;
 		}
 	} else
 		s_new->action = action;
@@ -894,14 +861,14 @@ static struct db_sys_list *_db_rule_gen_32(const struct arch_def *arch,
 	int chain_len_max;
 	struct db_sys_list *s_new;
 	struct db_arg_chain_tree *c_iter = NULL, *c_prev = NULL;
-	unsigned int tf_flag;
+	bool tf_flag;
 
 	s_new = malloc(sizeof(*s_new));
 	if (s_new == NULL)
 		return NULL;
 	memset(s_new, 0, sizeof(*s_new));
 	s_new->num = syscall;
-	s_new->valid = 1;
+	s_new->valid = true;
 	/* run through the argument chain */
 	chain_len_max = arch_arg_count_max(arch);
 	for (iter = 0; iter < chain_len_max; iter++) {
@@ -933,18 +900,18 @@ static struct db_sys_list *_db_rule_gen_32(const struct arch_def *arch,
 		switch (c_iter->op) {
 			case SCMP_CMP_NE:
 				c_iter->op = SCMP_CMP_EQ;
-				tf_flag = 0;
+				tf_flag = false;
 				break;
 			case SCMP_CMP_LT:
 				c_iter->op = SCMP_CMP_GE;
-				tf_flag = 0;
+				tf_flag = false;
 				break;
 			case SCMP_CMP_LE:
 				c_iter->op = SCMP_CMP_GT;
-				tf_flag = 0;
+				tf_flag = false;
 				break;
 			default:
-				tf_flag = 1;
+				tf_flag = true;
 		}
 
 		/* fixup the mask/datum */
@@ -955,10 +922,10 @@ static struct db_sys_list *_db_rule_gen_32(const struct arch_def *arch,
 	if (c_iter != NULL) {
 		/* set the leaf node */
 		if (tf_flag) {
-			c_iter->act_t_flg = 1;
+			c_iter->act_t_flg = true;
 			c_iter->act_t = action;
 		} else {
-			c_iter->act_f_flg = 1;
+			c_iter->act_f_flg = true;
 			c_iter->act_f = action;
 		}
 	} else
@@ -994,7 +961,7 @@ int db_rule_add(struct db_filter *db, uint32_t action, unsigned int syscall,
 	struct db_sys_list *s_new, *s_iter, *s_prev = NULL;
 	struct db_arg_chain_tree *c_iter = NULL, *c_prev = NULL;
 	struct db_arg_chain_tree *ec_iter, *ec_iter_b;
-	unsigned int rm_flag = 0;
+	bool rm_flag = false;
 	unsigned int new_chain_cnt = 0;
 	unsigned int n_cnt;
 
@@ -1042,7 +1009,7 @@ add_reset:
 		}
 		return 0;
 	} else if (s_iter->chains == NULL) {
-		if (rm_flag || s_iter->valid == 0) {
+		if (rm_flag || !s_iter->valid) {
 			/* we are here because our previous pass cleared the
 			 * entire syscall chain when searching for a subtree
 			 * match or the existing syscall entry is a phantom,
@@ -1050,9 +1017,9 @@ add_reset:
 			s_iter->chains = s_new->chains;
 			s_iter->action = s_new->action;
 			s_iter->node_cnt = s_new->node_cnt;
-			if (s_iter->valid == 1)
+			if (s_iter->valid)
 				s_iter->priority = s_new->priority;
-			s_iter->valid = 1;
+			s_iter->valid = true;
 			free(s_new);
 			rc = 0;
 			goto add_priority_update;
@@ -1076,12 +1043,11 @@ add_reset:
 		/* check for sub-tree matches in the existing tree */
 		rc = _db_tree_sub_prune(&(s_iter->chains), ec_iter, c_iter,
 					&rm_flag);
-		if (rc > 0)
+		if (rc > 0) {
 			s_iter->node_cnt -= rc;
-		else if (rc < 0)
-			goto add_free;
-		if (rm_flag)
 			goto add_reset;
+		} else if (rc < 0)
+			goto add_free;
 
 		/* check for sub-tree matches in the new tree */
 		ec_iter_b = ec_iter;
@@ -1095,10 +1061,7 @@ add_reset:
 		if (rc > 0) {
 			s_new->node_cnt -= rc;
 			if (s_new->node_cnt > 0)
-				/* XXX - are we ever going to hit this is a
-				 *       normal use case?  pretty sure we can't
-				 *       "reset" in this case ... */
-				return -EFAULT;
+				goto add_reset;
 			rc = 0;
 			goto add_free;
 		} else if (rc < 0)
@@ -1113,23 +1076,21 @@ add_reset:
 					if (ec_iter->act_t != action)
 						goto add_free_exist;
 				} else if (c_iter->act_t_flg) {
-					ec_iter->act_t_flg = 1;
+					ec_iter->act_t_flg = true;
 					ec_iter->act_t = action;
 				}
 				if (c_iter->act_f_flg && ec_iter->act_f_flg) {
 					if (ec_iter->act_f != action)
 						goto add_free_exist;
 				} else if (c_iter->act_f_flg) {
-					ec_iter->act_f_flg = 1;
+					ec_iter->act_f_flg = true;
 					ec_iter->act_f = action;
 				}
 				if (ec_iter->act_t_flg == ec_iter->act_f_flg &&
 				    ec_iter->act_t == ec_iter->act_f) {
 					n_cnt = _db_tree_remove(
 							     &(s_iter->chains),
-							     ec_iter,
-							     1,
-							     ec_iter->act_t);
+							     ec_iter);
 					s_iter->node_cnt -= n_cnt;
 				}
 				goto add_free_ok;
@@ -1142,7 +1103,7 @@ add_reset:
 						goto add_free;
 					n_cnt = _db_tree_free(ec_iter->nxt_t);
 					ec_iter->nxt_t = NULL;
-					ec_iter->act_t_flg = 1;
+					ec_iter->act_t_flg = true;
 					ec_iter->act_t = action;
 				} else {
 					rc = _db_tree_act_check(ec_iter->nxt_f,
@@ -1151,7 +1112,7 @@ add_reset:
 						goto add_free;
 					n_cnt = _db_tree_free(ec_iter->nxt_f);
 					ec_iter->nxt_f = NULL;
-					ec_iter->act_f_flg = 1;
+					ec_iter->act_f_flg = true;
 					ec_iter->act_f = action;
 				}
 				s_iter->node_cnt -= n_cnt;
