@@ -24,10 +24,12 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdbool.h>
 
 #include <seccomp.h>
 
 #include "arch.h"
+#include "arch-x32.h"
 #include "gen_bpf.h"
 #include "db.h"
 #include "hash.h"
@@ -53,7 +55,6 @@ enum bpf_jump_type {
 };
 
 struct bpf_jump {
-	enum bpf_jump_type type;
 	union {
 		uint8_t imm_j;
 		uint32_t imm_k;
@@ -62,21 +63,22 @@ struct bpf_jump {
 		struct bpf_blk *blk;
 		unsigned int nxt;
 	} tgt;
+	enum bpf_jump_type type;
 };
 #define _BPF_JMP_NO \
-	((struct bpf_jump) { TGT_NONE })
+	((struct bpf_jump) { .type = TGT_NONE })
 #define _BPF_JMP_NXT(x) \
-	((struct bpf_jump) { TGT_NXT, { .nxt = (x) } })  /* be careful! */
+	((struct bpf_jump) { .type = TGT_NXT, .tgt = { .nxt = (x) } })
 #define _BPF_JMP_IMM(x) \
-	((struct bpf_jump) { TGT_IMM, { .imm_j = (x) } })
+	((struct bpf_jump) { .type = TGT_IMM, .tgt = { .imm_j = (x) } })
 #define _BPF_JMP_DB(x) \
-	((struct bpf_jump) { TGT_PTR_DB, { .db = (x) } })
+	((struct bpf_jump) { .type = TGT_PTR_DB, .tgt = { .db = (x) } })
 #define _BPF_JMP_BLK(x) \
-	((struct bpf_jump) { TGT_PTR_BLK, { .blk = (x) } })
+	((struct bpf_jump) { .type = TGT_PTR_BLK, .tgt = { .blk = (x) } })
 #define _BPF_JMP_HSH(x) \
-	((struct bpf_jump) { TGT_PTR_HSH, { .hash = (x) } })
+	((struct bpf_jump) { .type = TGT_PTR_HSH, .tgt = { .hash = (x) } })
 #define _BPF_K(x) \
-	((struct bpf_jump) { TGT_K, { .imm_k = (x) } })
+	((struct bpf_jump) { .type = TGT_K, .tgt = { .imm_k = (x) } })
 #define _BPF_JMP_MAX			255
 #define _BPF_JMP_MAX_RET		255
 
@@ -97,30 +99,37 @@ struct bpf_blk {
 	/* priority - higher is better */
 	unsigned int priority;
 
+	/* status flags */
+	bool flag_hash;			/* added to the hash table */
+	bool flag_dup;			/* duplicate block and in use */
+	bool flag_unique;		/* ->blks is unique to this block */
+
 	/* original db_arg_chain_tree node */
 	const struct db_arg_chain_tree *node;
 
 	/* used during block assembly */
-	unsigned int hashed_flag;
 	uint64_t hash;
 	struct bpf_blk *hash_nxt;
 	struct bpf_blk *prev, *next;
 	struct bpf_blk *lvl_prv, *lvl_nxt;
+	struct acc_state acc_state;
 };
 #define _BLK_MSZE(x) \
 	((x)->blk_cnt * sizeof(*((x)->blks)))
 
 struct bpf_hash_bkt {
 	struct bpf_blk *blk;
-	unsigned int found;
-
 	struct bpf_hash_bkt *next;
+	unsigned int found;
 };
 
 #define _BPF_HASH_BITS			8
 #define _BPF_HASH_SIZE			(1 << _BPF_HASH_BITS)
 #define _BPF_HASH_MASK			(_BPF_HASH_BITS - 1)
 struct bpf_state {
+	/* block hash table */
+	struct bpf_hash_bkt *htbl[_BPF_HASH_SIZE];
+
 	/* filter attributes */
 	const struct db_filter_attr *attr;
 	/* default action */
@@ -129,9 +138,6 @@ struct bpf_state {
 	/* target arch - NOTE: be careful, temporary use only! */
 	const struct arch_def *arch;
 
-	/* block hash table */
-	struct bpf_hash_bkt *htbl[_BPF_HASH_SIZE];
-
 	/* bpf program */
 	struct bpf_program *bpf;
 };
@@ -159,7 +165,8 @@ struct bpf_state {
 static struct bpf_blk *_gen_bpf_chain(struct bpf_state *state,
 				      const struct db_sys_list *sys,
 				      const struct db_arg_chain_tree *chain,
-				      const struct bpf_jump *nxt_jump);
+				      const struct bpf_jump *nxt_jump,
+				      struct acc_state *a_state);
 
 static struct bpf_blk *_hsh_remove(struct bpf_state *state, uint64_t h_val);
 static struct bpf_blk *_hsh_find(const struct bpf_state *state, uint64_t h_val);
@@ -181,9 +188,10 @@ static void __blk_free(struct bpf_state *state, struct bpf_blk *blk)
 	while (blk->hash_nxt != NULL) {
 		b_tmp = blk->hash_nxt;
 		blk->hash_nxt = b_tmp->hash_nxt;
-		free(b_tmp);
+		if (!b_tmp->flag_dup)
+			free(b_tmp);
 	}
-	if (blk->blks != NULL)
+	if (blk->blks != NULL && blk->flag_unique)
 		free(blk->blks);
 	free(blk);
 }
@@ -263,6 +271,7 @@ static struct bpf_blk *_blk_append(struct bpf_state *state,
 		if (blk == NULL)
 			return NULL;
 		memset(blk, 0, sizeof(*blk));
+		blk->flag_unique = true;
 	}
 	if ((blk->blk_cnt + 1) > blk->blk_alloc) {
 		blk->blk_alloc += AINC_BLK;
@@ -278,51 +287,6 @@ static struct bpf_blk *_blk_append(struct bpf_state *state,
 	return blk;
 }
 
-#if 0 /* see NOTE in function description */
-/**
- * Append a BPF instruction block to another BPF instruction block
- * @param state the BPF state
- * @param dst the destination instruction block
- * @param src the source instruction block
- *
- * Append the source instruction block to the end of the destination block.
- * Returns a pointer to dst on success, NULL on failure, and in the case of
- * failure the destination block is free'd.
- *
- * NOTE: We're going to #if'def this function out and not simply remove it
- *	 because the need for such a function does seem to reoccur from time
- *	 to time and I don't feel like re-implementing it again in the near
- *	 future.  Once we've given the BPF generator a good scrubbing we can
- *	 remove this function if it still isn't used.
- *
- */
-static struct bpf_blk *_blk_append_blk(struct bpf_state *state,
-				       struct bpf_blk *dst,
-				       const struct bpf_blk *src)
-{
-	unsigned int size;
-	struct bpf_instr *new;
-
-	size = dst->blk_cnt + src->blk_cnt;
-	if (size > dst->blk_alloc) {
-		dst->blk_alloc = size;
-		new = realloc(dst->blks, dst->blk_alloc * sizeof(*(dst->blks)));
-		if (new == NULL) {
-			_blk_free(state, dst);
-			return NULL;
-		}
-		dst->blks = new;
-	}
-	memcpy(&dst->blks[dst->blk_cnt], src->blks,
-	       src->blk_cnt * sizeof(*(src->blks)));
-	dst->blk_cnt += src->blk_cnt;
-	if (src->priority > dst->priority)
-		dst->priority = src->priority;
-
-	return dst;
-}
-#endif /* see NOTE in function description */
-
 /**
  * Append a block of BPF instructions to the final BPF program
  * @param prg the BPF program
@@ -470,7 +434,7 @@ static int _hsh_add(struct bpf_state *state, struct bpf_blk **blk_p,
 	struct bpf_blk *blk = *blk_p;
 	struct bpf_blk *b_iter;
 
-	if (blk->hashed_flag)
+	if (blk->flag_hash)
 		return 0;
 
 	h_new = malloc(sizeof(*h_new));
@@ -481,7 +445,7 @@ static int _hsh_add(struct bpf_state *state, struct bpf_blk **blk_p,
 	/* generate the hash */
 	h_val = jhash(blk->blks, _BLK_MSZE(blk), 0);
 	blk->hash = h_val;
-	blk->hashed_flag = 1;
+	blk->flag_hash = true;
 	blk->node = NULL;
 	h_new->blk = blk;
 	h_new->found = (found ? 1 : 0);
@@ -497,6 +461,19 @@ static int _hsh_add(struct bpf_state *state, struct bpf_blk **blk_p,
 				/* duplicate block */
 				free(h_new);
 
+				/* store the duplicate block */
+				b_iter = h_iter->blk;
+				while (b_iter->hash_nxt != NULL)
+					b_iter = b_iter->hash_nxt;
+				b_iter->hash_nxt = blk;
+
+				/* in some cases we want to return the
+				 * duplicate block */
+				if (found) {
+					blk->flag_dup = true;
+					return 0;
+				}
+
 				/* update the priority if needed */
 				if (h_iter->blk->priority < blk->priority)
 					h_iter->blk->priority = blk->priority;
@@ -504,19 +481,15 @@ static int _hsh_add(struct bpf_state *state, struct bpf_blk **blk_p,
 				/* try to save some memory */
 				free(blk->blks);
 				blk->blks = h_iter->blk->blks;
+				blk->flag_unique = false;
 
-				/* store the duplicate block */
-				b_iter = h_iter->blk;
-				while (b_iter->hash_nxt != NULL)
-					b_iter = b_iter->hash_nxt;
-				b_iter->hash_nxt = blk;
 				*blk_p = h_iter->blk;
 				return 0;
 			} else if (h_iter->blk->hash == h_val) {
 				/* hash collision */
 				if ((h_val >> 32) == 0xffffffff) {
 					/* overflow */
-					blk->hashed_flag = 0;
+					blk->flag_hash = false;
 					blk->hash = 0;
 					return -EFAULT;
 				}
@@ -702,6 +675,7 @@ static struct bpf_blk *_gen_bpf_node(struct bpf_state *state,
 	uint64_t act_t_hash = 0, act_f_hash = 0;
 	struct bpf_blk *blk = NULL, *b_act;
 	struct bpf_instr instr;
+	struct acc_state a_state_orig = *a_state;
 
 	/* generate the action blocks */
 	if (node->act_t_flg) {
@@ -785,6 +759,7 @@ static struct bpf_blk *_gen_bpf_node(struct bpf_state *state,
 		goto node_failure;
 
 	blk->node = node;
+	blk->acc_state = a_state_orig;
 	return blk;
 
 node_failure:
@@ -816,7 +791,7 @@ static struct bpf_blk *_gen_bpf_chain_lvl_res(struct bpf_state *state,
 	struct bpf_instr *i_iter;
 	struct db_arg_chain_tree *node;
 
-	if (blk->hashed_flag)
+	if (blk->flag_hash)
 		return blk;
 
 	/* convert TGT_PTR_DB to TGT_PTR_HSH references */
@@ -838,7 +813,8 @@ static struct bpf_blk *_gen_bpf_chain_lvl_res(struct bpf_state *state,
 			break;
 		case TGT_PTR_DB:
 			node = (struct db_arg_chain_tree *)i_iter->jt.tgt.db;
-			b_new = _gen_bpf_chain(state, sys, node, nxt_jump);
+			b_new = _gen_bpf_chain(state, sys, node,
+					       nxt_jump, &blk->acc_state);
 			if (b_new == NULL)
 				return NULL;
 			i_iter->jt = _BPF_JMP_HSH(b_new->hash);
@@ -863,7 +839,8 @@ static struct bpf_blk *_gen_bpf_chain_lvl_res(struct bpf_state *state,
 			break;
 		case TGT_PTR_DB:
 			node = (struct db_arg_chain_tree *)i_iter->jf.tgt.db;
-			b_new = _gen_bpf_chain(state, sys, node, nxt_jump);
+			b_new = _gen_bpf_chain(state, sys, node,
+					       nxt_jump, &blk->acc_state);
 			if (b_new == NULL)
 				return NULL;
 			i_iter->jf = _BPF_JMP_HSH(b_new->hash);
@@ -898,6 +875,7 @@ static struct bpf_blk *_gen_bpf_chain_lvl_res(struct bpf_state *state,
  * @param sys the syscall filter
  * @param chain the filter chain
  * @param nxt_jump the jump to fallthrough to at the end of the level
+ * @param a_state the accumulator state
  *
  * Generate the BPF instruction blocks for the given filter chain and return
  * a pointer to the first block on success; returns NULL on failure.
@@ -906,13 +884,13 @@ static struct bpf_blk *_gen_bpf_chain_lvl_res(struct bpf_state *state,
 static struct bpf_blk *_gen_bpf_chain(struct bpf_state *state,
 				      const struct db_sys_list *sys,
 				      const struct db_arg_chain_tree *chain,
-				      const struct bpf_jump *nxt_jump)
+				      const struct bpf_jump *nxt_jump,
+				      struct acc_state *a_state)
 {
 	struct bpf_blk *b_head = NULL, *b_tail = NULL;
 	struct bpf_blk *b_prev, *b_next, *b_iter;
 	struct bpf_instr *i_iter;
 	const struct db_arg_chain_tree *c_iter;
-	struct acc_state a_state = { -1, ARG_MASK_MAX };
 	unsigned int iter;
 
 	if (chain == NULL) {
@@ -928,7 +906,7 @@ static struct bpf_blk *_gen_bpf_chain(struct bpf_state *state,
 
 		/* build all of the blocks for this level */
 		do {
-			b_iter = _gen_bpf_node(state, c_iter, &a_state);
+			b_iter = _gen_bpf_node(state, c_iter, a_state);
 			if (b_iter == NULL)
 				goto chain_failure;
 			if (b_head != NULL) {
@@ -1010,7 +988,7 @@ chain_failure:
  * @param state the BPF state
  * @param sys the syscall filter DB entry
  * @param nxt_hash the hash value of the next syscall filter DB entry
- * @param first this is the first syscall for the architecture
+ * @param acc_reset accumulator reset flag
  *
  * Generate the BPF instruction blocks for the given syscall filter and return
  * a pointer to the first block on success; returns NULL on failure.  It is
@@ -1021,27 +999,39 @@ chain_failure:
  */
 static struct bpf_blk *_gen_bpf_syscall(struct bpf_state *state,
 					const struct db_sys_list *sys,
-					uint64_t nxt_hash, int first)
+					uint64_t nxt_hash,
+					bool acc_reset)
 {
 	int rc;
 	struct bpf_instr instr;
 	struct bpf_blk *blk_c, *blk_s = NULL;
-	struct bpf_jump def_jump = _BPF_JMP_HSH(state->def_hsh);
+	struct bpf_jump def_jump;
+	struct acc_state a_state;
 
-	/* generate the argument chains */
-	blk_c = _gen_bpf_chain(state, sys, sys->chains, &def_jump);
-	if (blk_c == NULL)
-		return NULL;
+	/* we do the memset before the assignment to keep valgrind happy */
+	memset(&def_jump, 0, sizeof(def_jump));
+	def_jump = _BPF_JMP_HSH(state->def_hsh);
 
-	/* load the syscall into the accumulator (if needed) */
-	if (first) {
+	/* setup the accumulator state */
+	if (acc_reset) {
 		_BPF_INSTR(instr, BPF_LD+BPF_ABS, _BPF_JMP_NO, _BPF_JMP_NO,
 			_BPF_SYSCALL);
 		blk_s = _blk_append(state, NULL, &instr);
 		if (blk_s == NULL)
 			return NULL;
+		a_state.offset = _BPF_OFFSET_SYSCALL;
+		a_state.mask = ARG_MASK_MAX;
+	} else {
+		/* set the accumulator state to an unknown value */
+		a_state.offset = -1;
+		a_state.mask = ARG_MASK_MAX;
 	}
 
+	/* generate the argument chains */
+	blk_c = _gen_bpf_chain(state, sys, sys->chains, &def_jump, &a_state);
+	if (blk_c == NULL)
+		return NULL;
+
 	/* syscall check */
 	_BPF_INSTR(instr, BPF_JMP+BPF_JEQ,
 		   _BPF_JMP_HSH(blk_c->hash), _BPF_JMP_HSH(nxt_hash),
@@ -1065,18 +1055,21 @@ static struct bpf_blk *_gen_bpf_syscall(struct bpf_state *state,
  * Generate the BPF instruction blocks for a given filter/architecture
  * @param state the BPF state
  * @param db the filter DB
+ * @param db_secondary the secondary DB
  *
- * Generate the BPF instruction block for the given filter DB/architecture and
- * return a pointer to the block on succes, NULL on failure.  The resulting
+ * Generate the BPF instruction block for the given filter DB(s)/architecture(s)
+ * and return a pointer to the block on succes, NULL on failure.  The resulting
  * block assumes that the architecture token has already been loaded into the
  * BPF accumulator.
  *
  */
 static struct bpf_blk *_gen_bpf_arch(struct bpf_state *state,
-				     const struct db_filter *db)
+				     const struct db_filter *db,
+				     const struct db_filter *db_secondary)
 {
 	int rc;
 	unsigned int blk_cnt = 0;
+	bool acc_reset;
 	struct bpf_instr instr;
 	struct db_sys_list *s_head = NULL, *s_tail = NULL, *s_iter, *s_iter_b;
 	struct bpf_blk *b_head = NULL, *b_tail = NULL, *b_iter, *b_new;
@@ -1114,17 +1107,55 @@ static struct bpf_blk *_gen_bpf_arch(struct bpf_state *state,
 			s_head->pri_nxt = NULL;
 		}
 	}
+	if (db_secondary != NULL) {
+		db_list_foreach(s_iter, db_secondary->syscalls) {
+			if (s_head != NULL) {
+				s_iter_b = s_head;
+				while ((s_iter_b->pri_nxt != NULL) &&
+				(s_iter->priority <= s_iter_b->priority))
+					s_iter_b = s_iter_b->pri_nxt;
+
+				if (s_iter->priority > s_iter_b->priority) {
+					s_iter->pri_prv = s_iter_b->pri_prv;
+					s_iter->pri_nxt = s_iter_b;
+					if (s_iter_b == s_head) {
+						s_head->pri_prv = s_iter;
+						s_head = s_iter;
+					} else {
+						s_iter->pri_prv->pri_nxt=s_iter;
+						s_iter->pri_nxt->pri_prv=s_iter;
+					}
+				} else {
+					s_iter->pri_prv = s_tail;
+					s_iter->pri_nxt = NULL;
+					s_iter->pri_prv->pri_nxt = s_iter;
+					s_tail = s_iter;
+				}
+			} else {
+				s_head = s_iter;
+				s_tail = s_iter;
+				s_head->pri_prv = NULL;
+				s_head->pri_nxt = NULL;
+			}
+		}
+	}
+
+	if ((db->arch->token == SCMP_ARCH_X86_64 ||
+	     db->arch->token == SCMP_ARCH_X32) && (db_secondary == NULL))
+		acc_reset = false;
+	else
+		acc_reset = true;
 
 	/* create the syscall filters and add them to block list group */
 	for (s_iter = s_tail; s_iter != NULL; s_iter = s_iter->pri_prv) {
-		if (s_iter->valid == 0)
+		if (!s_iter->valid)
 			continue;
 
 		/* build the syscall filter */
 		b_new = _gen_bpf_syscall(state, s_iter,
 					 (b_head == NULL ?
 					  state->def_hsh : b_head->hash),
-					 (s_iter == s_head ? 1 : 0));
+					 (s_iter == s_head ? acc_reset:false));
 		if (b_new == NULL)
 			goto arch_failure;
 
@@ -1144,27 +1175,62 @@ static struct bpf_blk *_gen_bpf_arch(struct bpf_state *state,
 		blk_cnt++;
 	}
 
-	/* do the architecture check and load the syscall number */
-	if (b_head != NULL) {
-		/* arch check */
-		_BPF_INSTR(instr, BPF_JMP+BPF_JEQ,
-			   _BPF_JMP_HSH(b_head->hash),
-			   _BPF_JMP_NXT(blk_cnt),
-			   _BPF_K(db->arch->token));
-		b_head->prev = _blk_append(state, NULL, &instr);
-		if (b_head->prev == NULL)
+	/* additional ABI filtering */
+	if ((db->arch->token == SCMP_ARCH_X86_64 ||
+	     db->arch->token == SCMP_ARCH_X32) && (db_secondary == NULL)) {
+		_BPF_INSTR(instr, BPF_LD+BPF_ABS, _BPF_JMP_NO, _BPF_JMP_NO,
+			   _BPF_SYSCALL);
+		b_new = _blk_append(state, NULL, &instr);
+		if (b_new == NULL)
 			goto arch_failure;
-		b_head->prev->next = b_head;
-		b_head = b_head->prev;
-	} else {
-		/* arch check */
-		_BPF_INSTR(instr, BPF_JMP+BPF_JEQ,
-			   _BPF_JMP_HSH(state->def_hsh), _BPF_JMP_NXT(0),
-			   _BPF_K(db->arch->token));
-		b_head = _blk_append(state, NULL, &instr);
-		if (b_head == NULL)
+		if (db->arch->token == SCMP_ARCH_X86_64) {
+			/* filter out x32 */
+			_BPF_INSTR(instr, BPF_JMP+BPF_JGE,
+				_BPF_JMP_NXT(blk_cnt++), _BPF_JMP_NO,
+				_BPF_K(X32_SYSCALL_BIT));
+			if (b_head != NULL)
+				instr.jf = _BPF_JMP_HSH(b_head->hash);
+			else
+				instr.jf = _BPF_JMP_HSH(state->def_hsh);
+		} else if (db->arch->token == SCMP_ARCH_X32) {
+			/* filter out x86_64 */
+			_BPF_INSTR(instr, BPF_JMP+BPF_JGE,
+				_BPF_JMP_NO, _BPF_JMP_NXT(blk_cnt++),
+				_BPF_K(X32_SYSCALL_BIT));
+			if (b_head != NULL)
+				instr.jt = _BPF_JMP_HSH(b_head->hash);
+			else
+				instr.jt = _BPF_JMP_HSH(state->def_hsh);
+		} else
+			/* we should never get here */
+			goto arch_failure;
+		b_new = _blk_append(state, b_new, &instr);
+		if (b_new == NULL)
+			goto arch_failure;
+		b_new->next = b_head;
+		if (b_head != NULL)
+			b_head->prev = b_new;
+		b_head = b_new;
+		rc = _hsh_add(state, &b_head, 1);
+		if (rc < 0)
 			goto arch_failure;
 	}
+
+	/* do the ABI/architecture check */
+	_BPF_INSTR(instr, BPF_JMP+BPF_JEQ,
+		   _BPF_JMP_NO, _BPF_JMP_NXT(blk_cnt++),
+		   _BPF_K(db->arch->token_bpf));
+	if (b_head != NULL)
+		instr.jt = _BPF_JMP_HSH(b_head->hash);
+	else
+		instr.jt = _BPF_JMP_HSH(state->def_hsh);
+	b_new = _blk_append(state, NULL, &instr);
+	if (b_new == NULL)
+		goto arch_failure;
+	b_new->next = b_head;
+	if (b_head != NULL)
+		b_head->prev = b_new;
+	b_head = b_new;
 	rc = _hsh_add(state, &b_head, 1);
 	if (rc < 0)
 		goto arch_failure;
@@ -1379,10 +1445,12 @@ static int _gen_bpf_build_bpf(struct bpf_state *state,
 	uint64_t h_val;
 	unsigned int res_cnt;
 	unsigned int jmp_len;
+	int arch_x86_64 = -1, arch_x32 = -1;
 	struct bpf_instr instr;
 	struct bpf_instr *i_iter;
 	struct bpf_blk *b_badarch, *b_default;
 	struct bpf_blk *b_head = NULL, *b_tail = NULL, *b_iter, *b_new, *b_jmp;
+	struct db_filter *db_secondary = NULL;
 
 	if (col->filter_cnt == 0)
 		return -EINVAL;
@@ -1417,7 +1485,28 @@ static int _gen_bpf_build_bpf(struct bpf_state *state,
 
 	/* generate the per-architecture filters */
 	for (iter = 0; iter < col->filter_cnt; iter++) {
-		b_new = _gen_bpf_arch(state, col->filters[iter]);
+		if (col->filters[iter]->arch->token == SCMP_ARCH_X86_64)
+			arch_x86_64 = iter;
+		if (col->filters[iter]->arch->token == SCMP_ARCH_X32)
+			arch_x32 = iter;
+	}
+	for (iter = 0; iter < col->filter_cnt; iter++) {
+		/* figure out the secondary arch filter mess */
+		if (iter == arch_x86_64) {
+			if (arch_x32 > iter)
+				db_secondary = col->filters[arch_x32];
+			else if (arch_x32 >= 0)
+				continue;
+		} else if (iter == arch_x32) {
+			if (arch_x86_64 > iter)
+				db_secondary = col->filters[arch_x86_64];
+			else if (arch_x86_64 >= 0)
+				continue;
+		} else
+			db_secondary = NULL;
+
+		/* create the filter for the architecture(s) */
+		b_new = _gen_bpf_arch(state, col->filters[iter], db_secondary);
 		if (b_new == NULL)
 			return -ENOMEM;
 		b_new->prev = b_tail;
