@@ -49,10 +49,14 @@ struct pfc_sys_list {
 static const char *_pfc_arch(const struct arch_def *arch)
 {
 	switch (arch->token) {
-	case AUDIT_ARCH_I386:
+	case SCMP_ARCH_X86:
 		return "x86";
-	case AUDIT_ARCH_X86_64:
+	case SCMP_ARCH_X86_64:
 		return "x86_64";
+	case SCMP_ARCH_X32:
+		return "x32";
+	case SCMP_ARCH_ARM:
+		return "arm";
 	default:
 		return "UNKNOWN";
 	}
@@ -201,10 +205,11 @@ static void _gen_pfc_syscall(const struct arch_def *arch,
 			     const struct db_sys_list *sys, FILE *fds)
 {
 	unsigned int sys_num = sys->num;
+	const char *sys_name = arch_syscall_resolve_num(arch, sys_num);
 
 	_indent(fds, 1);
-	fprintf(fds, "# filter for syscall #%d (priority: %d)\n",
-		sys_num, sys->priority);
+	fprintf(fds, "# filter for syscall \"%s\" (%d) [priority: %d]\n",
+		(sys_name ? sys_name : "UNKNOWN"), sys_num, sys->priority);
 	_indent(fds, 1);
 	fprintf(fds, "if ($syscall == %d)\n", sys_num);
 	if (sys->chains == NULL) {
@@ -261,11 +266,11 @@ static int _gen_pfc_arch(const struct db_filter_col *col,
 	}
 
 	fprintf(fds, "# filter for arch %s (%u)\n",
-		_pfc_arch(db->arch), db->arch->token);
-	fprintf(fds, "if ($arch == %u)\n", db->arch->token);
+		_pfc_arch(db->arch), db->arch->token_bpf);
+	fprintf(fds, "if ($arch == %u)\n", db->arch->token_bpf);
 	p_iter = p_head;
 	while (p_iter != NULL) {
-		if (p_iter->sys->valid == 0)
+		if (!p_iter->sys->valid)
 			continue;
 		_gen_pfc_syscall(db->arch, p_iter->sys, fds);
 		p_iter = p_iter->next;
